@@ -24,7 +24,7 @@ BEGIN {
     use Exporter ();
     our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
-    $VERSION = '1.2.7';
+    $VERSION = '1.3.0';
 
     use base qw(Exporter);
     %EXPORT_TAGS = ();    # eg: TAG => [ qw!name1 name2! ],
@@ -134,7 +134,7 @@ sub thaw {
 # returns array of boxes with OCR text
 
 sub boxes {
-    my ( $self, @boxes ) = @_;
+    my ($self) = @_;
 
     # Unfortunately, there seems to be a case (tested in t/31_ocropus_utf8.t)
     # where decode_entities doesn't work cleanly, so encode/decode to finally
@@ -144,50 +144,107 @@ sub boxes {
         encode_utf8( HTML::Entities::decode_entities( $self->{hocr} ) ) );
 
     if ( $hocr =~ /<body>([\s\S]*)<\/body>/xsm ) {
-        my $p = HTML::TokeParser->new( \$hocr );
-        my ( $x1, $y1, $x2, $y2, $text );
-        while ( my $token = $p->get_token ) {
-            if ( $token->[0] eq 'S' ) {
-                if (
-                        $token->[1] eq 'span'
-                    and defined $token->[2]{class}
-                    and (  $token->[2]{class} eq 'ocr_line'
-                        or $token->[2]{class} eq 'ocr_word'
-                        or $token->[2]{class} eq 'ocrx_word' )
-                    and defined $token->[2]{title}
-                    and $token->[2]{title} =~
-                    /bbox[ ](\d+)[ ](\d+)[ ](\d+)[ ](\d+)/xsm
-                  )
-                {
-                    ( $x1, $y1, $x2, $y2 ) = ( $1, $2, $3, $4 );
+        return parse_hocr($hocr);
+    }
+    return {
+        type => 'page',
+        bbox => [ 0, 0, $self->{w}, $self->{h} ],
+        text => $hocr
+    };
+}
+
+sub parse_hocr {
+    my ($hocr) = @_;
+    my $p = HTML::TokeParser->new( \$hocr );
+    my ( $lineid, $wordid ) = ( 1, 1 );
+    my ( %linedata, %worddata, $data, @boxes );
+    while ( my $token = $p->get_token ) {
+        if ( $token->[0] eq 'S' ) {
+            my ( $tag, %attrs ) = ( $token->[1], %{ $token->[2] } );
+            if ( $tag eq 'span' and defined $attrs{class} ) {
+                if ( new_hocr_data(%attrs) ) {
+
+                    # initialize data pointer
+                    if ( $attrs{class} =~ /_word$/xsm ) {
+                        %worddata       = %linedata;
+                        $data           = \%worddata;
+                        $data->{type}   = 'word';
+                        $data->{lineid} = $linedata{id};
+                        $data->{id}     = $wordid++;
+                    }
+                    else {
+                        %linedata     = ();
+                        $data         = \%linedata;
+                        $data->{type} = 'line';
+                        $data->{id}   = $lineid++;
+                    }
+                    parse_tag_data( $attrs{title}, $data );
                 }
-                elsif ( $token->[1] eq 'span'
-                    and defined $token->[2]{class}
-                    and $token->[2]{class} eq 'ocr_cinfo' )
-                {
-                    undef $x1;
-                    undef $text;
+                elsif ( $attrs{class} eq 'ocr_cinfo' ) {
+
+                    # reset data pointer
+                    undef $data;
+                    %worddata = ();
+                    %linedata = ();
                 }
             }
-            if ( $token->[0] eq 'T' and $token->[1] !~ /^\s*$/xsm ) {
-                $text = $token->[1];
-                chomp $text;
-            }
-            if ( $token->[0] eq 'E' ) {
-                undef $x1;
-                undef $text;
-            }
-            if ( defined $x1 and defined $text ) {
-                push @boxes, [ $x1, $y1, $x2, $y2, $text ];
+            elsif ( defined $data ) {
+                if ( $tag eq 'strong' ) { push @{ $data->{style} }, 'Bold' }
+                if ( $tag eq 'em' )     { push @{ $data->{style} }, 'Italic' }
             }
         }
-    }
-    else {
-        push @boxes, [ 0, 0, $self->{w}, $self->{h}, $hocr ];
+        elsif ( $token->[0] eq 'T' ) {
+            if ( $token->[1] !~ /^\s*$/xsm ) {
+                $data->{text} = $token->[1];
+                chomp $data->{text};
+            }
+        }
+        elsif ( $token->[0] eq 'E' ) {
+
+            # reset data pointer
+            undef $data;
+        }
+
+        if (    defined $data
+            and defined $data->{text}
+            and defined $data->{bbox} )
+        {
+            push @boxes, { %{$data} };
+        }
     }
     return @boxes;
 }
 
+sub new_hocr_data {
+    my %attrs = @_;
+    return (
+        (
+                 $attrs{class} eq 'ocr_line'
+              or $attrs{class} eq 'ocr_word'
+              or $attrs{class} eq 'ocrx_word'
+        )
+          and defined $attrs{title}
+    );
+}
+
+sub parse_tag_data {
+    my ( $title, $data ) = @_;
+    if ( $title =~ /\bbbox\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/xsm ) {
+        $data->{bbox} = [ $1, $2, $3, $4 ];
+    }
+    if ( $title =~ /\btextangle\s+(\d+)/xsm ) { $data->{textangle}  = $1 }
+    if ( $title =~ /\bx_wconf\s+(\d+)/xsm )   { $data->{confidence} = $1 }
+    if ( $title =~ /\bbaseline\s+(?:-?\d+(?:[.]\d+)?\s+)*(-?\d+)/xsm ) {
+        $data->{baseline} = $1;
+
+        # slightly kludgy: textangle defined means 90°
+        $data->{base} = $data->{baseline} + defined $data->{textangle}
+          ? $data->{bbox}[3]    ## no critic (ProhibitMagicNumbers)
+          : $data->{bbox}[2];
+    }
+    return;
+}
+
 sub to_png {
     my ( $self, $page_sizes ) = @_;
 
