@@ -28,7 +28,7 @@ use Glib::Object::Subclass Gtk2::VBox::,
     ),
   ];
 
-our $VERSION = '1.2.7';
+our $VERSION = '1.3.0';
 
 my @widget_list;
 
@@ -45,7 +45,7 @@ sub INIT_INSTANCE {
     #the first radio button has to set the group,
     #which is undef for the first button
     my $group;
-    foreach my $nick ( keys %buttons ) {
+    foreach my $nick ( sort keys %buttons ) {
         $self->{button}{$nick} =
           Gtk2::RadioButton->new( $group, $buttons{$nick} );
         $self->{button}{$nick}->signal_connect(
